#!/usr/bin/env python
#
# xiutilities documentation build configuration file, created by
# sphinx-quickstart on Fri Jun  9 13:47:02 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another
# directory, add these directories to sys.path here. If the directory is
# relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('..'))
sys.path.append(os.path.abspath('../..'))

import xiutilities

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.viewcode',
    'sphinx_rtd_theme',
    'sphinx_favicon',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'xiUtils'
copyright = "2024, Rappsilber Laboratory"
author = "Falk Boudewijn Schimweg"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = xiutilities.__version__
# The full version, including alpha/beta/rc tags.
release = xiutilities.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#
#html_theme_options = {
#  'logo_only': True,
#}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

#favicons = [
#  {"href": "logo.svg"},
#]

#html_logo = "_static/logo-slim.svg"

# -- Options for HTMLHelp output ---------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'xiutilitiesdoc'


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'xiutilities.tex',
     'xiUtils Documentation',
     'Falk Boudewijn Schimweg', 'manual'),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'xiutilities',
     'xiUtils Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'xiutilities',
     'xiUtils Documentation',
     author,
     'xiutilities',
     'A collection of corss-link mass spectrometry tools.',
     'Miscellaneous'),
]






import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sns
import xiutils.bi_fdr as fdr


def fdr_cutoff_plot(df, xlim=0.05, score_col='match_score', rescore_col='xiMLScore', fdr_steps=0.001,
                    top_col='top_ranking', top_col_rescored='xiMLScore_top_ranking',
                    native_fdr_col=None, rescore_fdr_col=None, mismatch_col=None, ylabel='CSMs',
                    print_points=[]):
    # Calculate FDR if none provided
    if native_fdr_col is None:
        df.drop(['fdr_native', 'fdr_rescored'], axis=1, inplace=True, errors='ignore')
        df.loc[:, 'fdr_native'] = fdr.calculate_bi_fdr(df, score_col=score_col)
    else:
        df.loc[:, 'fdr_native'] = df.loc[:, native_fdr_col]

    if rescore_fdr_col is None:
        df.drop(['fdr_native', 'fdr_rescored'], axis=1, inplace=True, errors='ignore')
        df.loc[:, 'fdr_rescored'] = fdr.calculate_bi_fdr(df, score_col=rescore_col)
    else:
        df.loc[:, 'fdr_rescored'] = df.loc[:, rescore_fdr_col]
    
    df = df[
        ((df['fdr_native'] <= xlim) | (df['fdr_rescored'] <= xlim)) & df['isTT']
    ]

    fdr_x = np.arange(0, xlim+fdr_steps, fdr_steps)

    samples_native_cumsum = pd.DataFrame(index=fdr_x)
    samples_rescored_cumsum = pd.DataFrame(index=fdr_x)
    samples_gained_cumsum = pd.DataFrame(index=fdr_x)
    samples_lost_cumsum = pd.DataFrame(index=fdr_x)
    rescore_mismatch_cumsum = pd.DataFrame(index=fdr_x)
    native_mismatch_cumsum = pd.DataFrame(index=fdr_x)
    
    # Calculate sample number for FDR cutoff
    native_below = df[
        (df['fdr_native'] <= xlim) &
        df['isTT']
    ]
    samples_native_cumsum['count'] = samples_native_cumsum.index.to_series().apply(
        lambda val: len(native_below[native_below['fdr_native'] <= val])
    )
    
    rescored_below = df[
        (df['fdr_rescored'] <= xlim) &
        df['isTT']
    ]
    samples_rescored_cumsum['count'] = samples_rescored_cumsum.index.to_series().apply(
        lambda val: len(rescored_below[rescored_below['fdr_rescored'] <= val])
    )
    
    # Calculate gain
    gain_cands = df[
        (df['fdr_rescored'] <= xlim) &
        df['isTT']
    ]
    samples_gained_cumsum['count'] = samples_gained_cumsum.index.to_series().apply(
        lambda val: len(gain_cands[
            (gain_cands['fdr_native'] > val) &
            (gain_cands['fdr_rescored'] <= val)
        ]) + len(rescored_below[
                (rescored_below['fdr_rescored'] <= val) &
                (~rescored_below[top_col]) & rescored_below[top_col_rescored]
        ])
    )

    # Calculate loss
    loss_cands = df[
        (df['fdr_native'] <= xlim) &
        df['isTT']
    ]
    samples_lost_cumsum['count'] = samples_lost_cumsum.index.to_series().apply(
        lambda val: len(loss_cands[
            (
                (loss_cands['fdr_native'] <= val) &
                (loss_cands['fdr_rescored'] > val)
            )
        ]) + len(native_below[
                (native_below['fdr_native'] <= val) &
                native_below[top_col] & (~native_below[top_col_rescored])
        ])
    )

    if mismatch_col is not None:
        # Calculate rescore mismatch 
        rescore_mismatch_cand = df[
            (df['fdr_rescored'] <= xlim) &
            df['isTT'] & df[mismatch_col]
        ]
        rescore_mismatch_cumsum['count'] = rescore_mismatch_cumsum.index.to_series().apply(
            lambda val: len(rescore_mismatch_cand[
                (rescore_mismatch_cand['fdr_rescored'] <= val)
            ])
        )

        # Calculate native mismatch
        native_mismatch_cand = df[
            (df['fdr_native'] <= xlim) &
            df['isTT'] & df[mismatch_col]
        ]
        native_mismatch_cumsum['count'] = native_mismatch_cumsum.index.to_series().apply(
            lambda val: len(native_mismatch_cand[
                (
                    (native_mismatch_cand['fdr_native'] <= val)
                )
            ])
        )
    
    fig, ax = plt.subplots(figsize=(12, 6), dpi=100)

    data = {
        'native': samples_native_cumsum['count'],
        'rescored': samples_rescored_cumsum['count'],
        'gained': samples_gained_cumsum['count'],
        'lost': samples_lost_cumsum['count']
    }
    
    sns.lineplot(
        ax=ax,
        data=data
    )
    
    plt.xlabel('FDR cutoff')
    plt.ylabel(ylabel)
    
    ax.set_xlim(0, xlim)
    xfilter = samples_native_cumsum.loc[:xlim].index
    
    ylim = max([
        samples_native_cumsum.loc[xfilter, 'count'].max(),
        samples_rescored_cumsum.loc[xfilter, 'count'].max()
    ])*1.1
    
    ax.set_ylim(
        0,
        ylim
    )
    
    for x in print_points:
        if x > fdr_x[-1]:
            continue
        x_idx = 0
        while x > fdr_x[x_idx] or x_idx >= len(fdr_x):
            x_idx += 1
        y_native = samples_native_cumsum['count'].iloc[x_idx]
        y_rescored = samples_rescored_cumsum['count'].iloc[x_idx]
        y = max([y_native, y_rescored])
        text = f"{y_native}→{y_rescored}"
        ax.plot(
            x,
            y,
            'xk'
        )
        plt.text(
            x=x,
            y=y+ylim*0.01,
            s=text
        )
    
    if mismatch_col is not None:
        data2 = {
            'native mismatch': native_mismatch_cumsum['count']/data['native']*100,
            'rescore mismatch': rescore_mismatch_cumsum['count']/data['rescored']*100,
            'fdr': fdr_x*100,
        }
        ax2 = ax.twinx()
        
        sns.lineplot(
            ax=ax2,
            data=data2
        )
    
    return fig


def fdr_mismatch_plot(df, xlim=0.05, score_col='match_score', rescore_col='xiMLScore', fdr_steps=0.001,
                      native_fdr_col=None, rescore_fdr_col=None, mismatch_col='mismatch', ylabel='proportion'):
    # Calculate FDR if none provided
    if native_fdr_col is None:
        df.drop(['fdr_native', 'fdr_rescored'], axis=1, inplace=True, errors='ignore')
        df.loc[:, 'fdr_native'] = fdr.calculate_bi_fdr(df, score_col=score_col)
    else:
        df.loc[:, 'fdr_native'] = df.loc[:, native_fdr_col]

    if rescore_fdr_col is None:
        df.drop(['fdr_native', 'fdr_rescored'], axis=1, inplace=True, errors='ignore')
        df.loc[:, 'fdr_rescored'] = fdr.calculate_bi_fdr(df, score_col=rescore_col)
    else:
        df.loc[:, 'fdr_rescored'] = df.loc[:, rescore_fdr_col]
    
    df = df.loc[
        (df['fdr_native'] <= xlim) | (df['fdr_rescored'] <= xlim)
    ]

    fdr_x = np.arange(0, xlim+fdr_steps, fdr_steps)

    fdr_series = pd.DataFrame(index=fdr_x, data={'fdr': fdr_x})
    samples_native_cumsum = pd.DataFrame(index=fdr_x)
    samples_rescored_cumsum = pd.DataFrame(index=fdr_x)
    rescore_mismatch_cumsum = pd.DataFrame(index=fdr_x)
    native_mismatch_cumsum = pd.DataFrame(index=fdr_x)
    
    # Calculate sample number for FDR cutoff
    native_below = df.loc[df['fdr_native'] <= xlim]
    samples_native_cumsum['count'] = samples_native_cumsum.index.to_series().apply(
        lambda val: len(native_below[native_below['fdr_native'] <= val])
    )
    
    rescored_below = df.loc[df['fdr_rescored'] <= xlim]
    samples_rescored_cumsum['count'] = samples_rescored_cumsum.index.to_series().apply(
        lambda val: len(rescored_below.loc[rescored_below['fdr_rescored'] <= val])
    )
    
    # Calculate rescore mismatch 
    rescore_mismatch_cand = df.loc[
        (df['fdr_rescored'] <= xlim) &
        df['isTT'] & df[mismatch_col]
    ]
    rescore_mismatch_cumsum.loc[:, 'count'] = rescore_mismatch_cumsum.index.to_series().apply(
        lambda val: len(rescore_mismatch_cand[
            (rescore_mismatch_cand['fdr_rescored'] <= val)
        ])
    )

    # Calculate native mismatch
    native_mismatch_cand = df.loc[
        (df['fdr_native'] <= xlim) &
        df['isTT'] & df[mismatch_col]
    ]
    native_mismatch_cumsum.loc[:, 'count'] = native_mismatch_cumsum.index.to_series().apply(
        lambda val: len(native_mismatch_cand[
            (
                (native_mismatch_cand['fdr_native'] <= val)
            )
        ])
    )
    
    fig, ax = plt.subplots(figsize=(12, 6), dpi=100)

    data = {
        'native mismatch': native_mismatch_cumsum['count']/samples_native_cumsum['count'],
        'rescore mismatch': rescore_mismatch_cumsum['count']/samples_rescored_cumsum['count'],
        'fdr': fdr_series['fdr'],
    }
    
    sns.lineplot(
        ax=ax,
        data=data
    )
    
    plt.xlabel('FDR cutoff')
    plt.ylabel(ylabel)
    
    ax.set_xlim(0, xlim)
    
    return fig, data



from functools import partial
import numpy as np
import multiprocess as mp
import os
from math import ceil
from time import sleep
import pandas as pd


def self_or_between(df: pd.DataFrame,
                    col_prot1: str = 'protein_p1',
                    col_prot2: str = 'protein_p2',
                    str_self: str = 'self',
                    str_between: str = 'between',
                    decoy_adj: str = 'REV_',
                    sep: str = ';') -> pd.Series:
    """
    Classify CSMs as self or between links based on a ``sep`` separated list using multiprocessing

    :param df: Pandas DataFrame with CSMs
    :param col_prot1: Name of column containing first protein names
    :param col_prot2: Name of column contianing second protein names
    :param str_self: String marking a self link
    :param str_between: String marking a between link
    :param decoy_adj: String marking a decoy protein (will be removed for classification)
    :param sep: String that separates the protein lists
    :return: Series of classifications
    :rtype: Series
    """
    df.loc[:, f'{col_prot1}_arr'] = df.loc[:, col_prot1]\
        .astype(str).str.replace(decoy_adj, '')\
        .str.split(sep).map(np.unique).map(list)
    df.loc[:, f'{col_prot2}_arr'] = df.loc[:, col_prot2]\
        .astype(str).str.replace(decoy_adj, '')\
        .str.split(sep).map(np.unique).map(list)
    df.loc[:, 'proteins_arr'] = (df.loc[:, f'{col_prot1}_arr'] + df.loc[:, f'{col_prot2}_arr'])
    df.loc[:, 'proteins_arr_unique'] = df.loc[:, 'proteins_arr'].map(np.unique)
    df.loc[:, 'is_between'] = ~(
            df.loc[:, 'proteins_arr'].map(len) - df.loc[:, 'proteins_arr_unique'].map(len)
    ).astype(bool)
    df.drop(
        [
            f'{col_prot1}_arr',
            f'{col_prot2}_arr',
            'proteins_arr',
            'proteins_arr_unique'
        ],
        axis=1,
        inplace=True
    )
    return df.loc[:, 'is_between'].map({True: str_between, False: str_self})


def self_or_between_mp(df: pd.DataFrame,
                       col_prot1: str = 'protein_p1',
                       col_prot2: str = 'protein_p2',
                       str_self: str = 'self',
                       str_between: str = 'between',
                       decoy_adj: str = 'REV_',
                       sep: str = ';') -> pd.Series:
    """
    Classify CSMs as self or between links based on a ``sep`` separated list

    :param df: Pandas DataFrame with CSMs
    :param col_prot1: Name of column containing first protein names
    :param col_prot2: Name of column contianing second protein names
    :param str_self: String marking a self link
    :param str_between: String marking a between link
    :param decoy_adj: String marking a decoy protein (will be removed for classification)
    :param sep: String that separates the protein lists
    :return: Series of classifications
    :rtype: Series
    """
    pool_size = min([10,os.cpu_count()])
    slice_size = ceil(len(df)/pool_size)
    cols = [col_prot1, col_prot2]
    df = df.loc[:, cols]
    df_slices = [
        df.iloc[i*slice_size:(i+1)*slice_size][cols]
        for i in range(pool_size)
    ]
    print('slicing done')
    print(f"Pool size: {pool_size}")
    with mp.Pool(processes=pool_size) as pool:
        job_self_between = partial(
            self_or_between,
            decoy_adj=decoy_adj,
            str_self=str_self,
            str_between=str_between,
            col_prot1=col_prot1,
            col_prot2=col_prot2,
            sep=sep,
        )
        map_res = pool.map(job_self_between, df_slices)
    return pd.concat(map_res).copy()


def calculate_fdr(df,
                  decoy_p1='decoy_p1',
                  decoy_p2='decoy_p2',
                  score_col='match_score',
                  decoy_class=None,
                  max_slices=None):
    orig_sorting = df.index
    df_by_score_desc = df.sort_values(score_col, ascending=False)

    if decoy_class is None:
        is_dd = df_by_score_desc.loc[:, decoy_p1] & df_by_score_desc.loc[:, decoy_p2]
        is_tt = (~df_by_score_desc.loc[:, decoy_p1]) & (~df_by_score_desc.loc[:, decoy_p2])
        is_td = (~is_tt) & (~is_dd)
    else:
        is_dd = df_by_score_desc[decoy_class] == 'DD'
        is_td = df_by_score_desc[decoy_class] == 'TD'
        is_tt = df_by_score_desc[decoy_class] == 'TT'

    n_slices = os.cpu_count()
    if max_slices is not None:
        n_slices = min([n_slices, max_slices])
    slice_size = ceil(len(df)/n_slices)

    with mp.Pool(10) as pool:
        # Calculate sliced cumsum
        tt_cumsum_ares = []
        td_cumsum_ares = []
        dd_cumsum_ares = []
        for i in range(n_slices):
            slice_tt = is_tt.iloc[i*slice_size : (i+1)*slice_size].to_numpy()
            slice_td = is_td.iloc[i*slice_size : (i+1)*slice_size].to_numpy()
            slice_dd = is_dd.iloc[i*slice_size : (i+1)*slice_size].to_numpy()

            tt_cumsum_ares.append(
                pool.apply_async(np.cumsum, (slice_tt,))
            )
            td_cumsum_ares.append(
                pool.apply_async(np.cumsum, (slice_td,))
            )
            dd_cumsum_ares.append(
                pool.apply_async(np.cumsum, (slice_dd,))
            )

        # Wait for cumsum slices
        for x in [tt_cumsum_ares, td_cumsum_ares, dd_cumsum_ares]:
            while np.count_nonzero([not r.ready() for r in x]) > 0:
                sleep(3)

        # Merge slices
        tt_cumsum = None
        for r in tt_cumsum_ares:
            if tt_cumsum is None or len(tt_cumsum) == 0:
                tt_cumsum = r.get()
            else:
                offset = tt_cumsum[-1]
                tt_cumsum = np.append(
                    tt_cumsum,
                    r.get() + offset
                )
        td_cumsum = None
        for r in td_cumsum_ares:
            if td_cumsum is None or len(td_cumsum) == 0:
                td_cumsum = r.get()
            else:
                offset = td_cumsum[-1]
                td_cumsum = np.append(
                    td_cumsum,
                    r.get() + offset
                )
        dd_cumsum = None
        for r in dd_cumsum_ares:
            if dd_cumsum is None or len(dd_cumsum) == 0:
                dd_cumsum = r.get()
            else:
                offset = dd_cumsum[-1]
                dd_cumsum = np.append(
                    dd_cumsum,
                    r.get() + offset
                )

        fdr_raw = (td_cumsum - dd_cumsum) / tt_cumsum
        fdr_raw_flip = np.flip(fdr_raw)

        ares = []
        # Spawn workers
        for i in range(n_slices):
            slice = pd.Series(
                fdr_raw_flip[i*slice_size : (i+1)*slice_size]
            )
            ares.append(
                pool.apply_async(
                    slice.cummin
                )
            )

        # Wait for cummin
        while np.count_nonzero([not r.ready() for r in ares]) > 0:
            sleep(3)

        fdr_flip = None
        # Merge slices
        for r in ares:
            if fdr_flip is None or len(fdr_flip)==0:
                fdr_flip = r.get().to_numpy()
            else:
                fdr_flip = np.append(
                    fdr_flip,
                    np.clip(
                        r.get(),
                        a_min=0,
                        a_max=fdr_flip[-1]
                    )
                )

        df_by_score_desc.loc[:, 'fdr'] = np.flip(fdr_flip)
        return df_by_score_desc.loc[orig_sorting, 'fdr']


def calculate_bi_fdr(df,
                     score_col='match_score',
                     fdr_group_col=None,
                     decoy_class=None,
                     max_slices=None,
                     decoy_p1='decoy_p1',
                     decoy_p2='decoy_p2',
                     str_self='self') -> pd.Series:
    """
    Calculate separate FDRs for self and between links

    :param df: DataFrame with CSMs
    :param score_col: Main score column
    :param fdr_group_col: Self/between column
    :param decoy_class: TT/TD/DD column
    :param max_slices: Limit of CPU cores
    :param decoy_p1: Decoy column for first peptide
    :param decoy_p2: Decoy column for second peptide
    :param str_self: String marking a self link
    :return: Series of FDRs
    :rtype: Series
    """
    original_order = df.index
    if fdr_group_col is None:
        df.loc[:,'fdr_group'] = self_or_between_mp(df)
    else:
        df.loc[:, 'fdr_group'] = df.loc[:, fdr_group_col]
    self = df.loc[:, 'fdr_group'] == str_self

    df_by_index = df.sort_index()
    self_fdr = calculate_fdr(
        df_by_index.loc[self],
        score_col=score_col,
        decoy_class=decoy_class,
        decoy_p1=decoy_p1,
        decoy_p2=decoy_p2,
        max_slices=max_slices
    )
    between_fdr = calculate_fdr(
        df_by_index.loc[~self],
        score_col=score_col,
        decoy_class=decoy_class,
        decoy_p1=decoy_p1,
        decoy_p2=decoy_p2,
        max_slices=max_slices
    )
    return pd.concat([self_fdr, between_fdr]).loc[original_order]



from importlib.metadata import version

from pkg_resources import DistributionNotFound

"""Top-level package for xiRescore."""

__author__ = """Falk Boudewijn Schimweg"""
__email__ = 'git@falk.schimweg.de'
try:
    __version__ = version('xiutilities')
except DistributionNotFound:
    # package is not installed
    __version__ = '0.0.0'



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "pandas>=2.2.2",
    "numpy>=1.26.4",
    "setuptools>=71.1.0",
    "numpy-indexed>=0.3.7",
    "tqdm>=4.66.0",
    "pyteomics>=4.6",
    "seaborn>=0.12.2",
    "pytest>=8.3.1",
    "pydocstyle>=6.3.0",
    "pytest-cov>=4.0.0",
    "pytest-flake8>=1.0.6",
    "pytest-pydocstyle>=2.3.2",
    "py>=1.11.0",
    "flake8==4.0.1",
    "networkx>=3.1",
    "multiprocess>=0.70.16",
    "deepmerge~=1.1.0",
    "fastparquet>=2024.5.0",
    "pyarrow>=17.0.0",
    "scipy>=1.12.0",
    "matplotlib",
]

test_requirements = ['pytest>=3', ]

setup(
    author="Falk Boudewijn Schimweg",
    author_email='git@falk.schimweg.de',
    python_requires='>=3.7',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="A collection of corss-link mass spectrometry tools.",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='xiutilities',
    name='xiutilities',
    packages=find_packages(include=['xiutilities', 'xiutilities.*'], exclude=["tests"]),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Rappsilber-Laboratory/xiUtils',
    zip_safe=False,
)
